﻿using Microsoft.AspNetCore.Mvc;
using RedisAndEntityFrameworkInWebApi.Data;
using StackExchange.Redis;
using System.Threading.Tasks;

namespace RedisAndEntityFrameworkInWebApi.Controllers
{
    [Route("[controller]")]
    [ApiController]
    public class ValuesController : ControllerBase
    {
        private readonly IConnectionMultiplexer _connectionMultiplexer;
        private readonly KeyAndValueContext _keyAndValueContext;

        public ValuesController(IConnectionMultiplexer connectionMultiplexer, KeyAndValueContext keyAndValueContext)
        {
            _connectionMultiplexer = connectionMultiplexer;
            _keyAndValueContext = keyAndValueContext;
        }

        [HttpGet("{key}")]
        public async Task<IActionResult> Get(string key)
        {
            var redis = _connectionMultiplexer.GetDatabase();
            var redisResponse = await redis.StringGetAsync(key);
            
            if(redisResponse.HasValue)
            {
                return Ok($"key:{key}, value:{redisResponse.ToString()}, source:Redis");
            }

            var dbResponse = await _keyAndValueContext.FindAsync<KeyAndValue>(key);
            await redis.StringSetAsync(key, dbResponse.Value); // optionally add the key to the Redis
            return Ok($"key:{key}, value:{dbResponse.Value}, source:MSSQL");
        }
    }
}