using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using StackExchange.Redis;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using RedisAndEntityFrameworkInWebApi.Data;

namespace RedisAndEntityFrameworkInWebApi
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddDbContext<KeyAndValueContext>(options =>
            options.UseSqlServer(Configuration["ConnectionStrings:SalesDb"]));
            var multiplexer = ConnectionMultiplexer.Connect("localhost");
            services.AddSingleton<IConnectionMultiplexer>(multiplexer);
            services.AddControllers();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env, KeyAndValueContext keyAndValueContext, IConnectionMultiplexer multiplexer)
        {
            keyAndValueContext.Database.EnsureDeleted();
            keyAndValueContext.Database.EnsureCreated();
            keyAndValueContext.SeedEntityFramework();

            var redis = multiplexer.GetDatabase();

            Console.WriteLine("Writing to the cache...");
            foreach (string letter in Seeder.Alphabet)
            {   
                redis.StringSet(letter, $"{letter}{letter}{letter}");
            }

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}
